#!/bin/sh

# This script first starts faked (the daemon), and then it will run
# the requested program with fake root privileges.

usage () {
cat - >&2 <<EOF
fakeroot, create a fake root environment.
   usage: fakeroot [-l|--lib fakerootlib] [-f|--faked fakedbin]
                   [-i file] [-s file] [-u|--unknown-is-real]
		   [-b|--fd-base fd] [-h|--help] [-v|--version]
                   [--] [command]
EOF
  exit 1
}

get_default_mode()
{
	local RESULT
	local CPU_TRANSPARENCY

	RESULT='sysv'

	if [ -L "$TARGET_CONFIG" ] ; then
		CPU_TRANSPARENCY=`grep SBOX_CPUTRANSPARENCY_METHOD "$TARGET_CONFIG" | cut -d = -f 2`
		if echo "$CPU_TRANSPARENCY" | grep -q sbrsh ; then
			RESULT='tcp'
		fi
	fi

	echo "$RESULT"
}

warn()
{
	[ "z$SBOX_FAKEROOT_VERBOSE" != 'z' ] && echo "fakeroot: $@" >&2
}

err()
{
	echo "fakeroot: $@" >&2
	exit 1
}


stderr ()
{
  local i
  for i
  do
      echo >&2 "fakeroot: $i"
  done
}

fatal ()
{
  stderr "$@"
  exit 1
}

# strip /bin/fakeroot to find install prefix
FAKEROOT_BINDIR=/scratchbox/tools/bin
TARGET_PREFIX=/usr
HOST_PREFIX=/scratchbox/tools

# scratchbox specific
TARGET_CONFIG='/targets/links/scratchbox.config'
DEVICE_TOOLS='/scratchbox/device_tools'

if [ ! -e "$TARGET_CONFIG" ] ; then
	err "I'm modified to be used exclusively from inside scratchbox"
fi

# decide over tcp and sysv
if [ "z$SBOX_FAKEROOT_MODE" = 'z' ] ; then
	SBOX_FAKEROOT_MODE=`get_default_mode`
fi

# find the fakeroot version supplied with the toolchain
TARGET_COMPILER=`cat "$TARGET_CONFIG" | grep ^SBOX_CROSS_GCC_NAME | cut -d = -f 2`

case "$SBOX_FAKEROOT_MODE" in
	sysv)
		;;
	tcp)
		;;
	*)
		warn "Unknown mode '$SBOX_FAKEROOT_MODE', resetting to default"
		SBOX_FAKEROOT_MODE=`get_default_mode`
		;;
esac

LIB_HOST="libfakeroot-host-${SBOX_FAKEROOT_MODE}.so"
LIB_TARGET="libfakeroot-${SBOX_FAKEROOT_MODE}.so"

LIB_PREFIX="${HOST_PREFIX}"
if [ "z$TARGET_COMPILER" != 'zhost-gcc' ] ; then
	LIB_PREFIX="${TARGET_PREFIX}"
else
	LIB_TARGET="$LIB_HOST"
fi

# backwards compability

FAKEROOT_LIBDIR=
if [ "z$TARGET_COMPILER" != 'z' ] && [ "z$TARGET_COMPILER" != 'zhost-gcc' ] ; then
	FAKEROOT_LIBDIR=$(for i in `find "$DEVICE_TOOLS" -maxdepth 1 -mindepth 1 -name fakeroot-\* -type d | sort -r` ; do if [ -e "${i}/${TARGET_COMPILER}" ] ; then echo "${i}/${TARGET_COMPILER}/lib" ; break ; fi ; done)
fi

if [ "z$FAKEROOT_LIBDIR" != 'z' ] ; then
	# fakeroot 1.4.2.1
	if [ -e "${FAKEROOT_LIBDIR}/libfakeroot-${SBOX_FAKEROOT_MODE}" ] ; then
		LIB_TARGET='libfakeroot.so.0'
	else
		# fakeroot 1.4.2 and older (no sysv variant)
		if [ ! -e "${FAKEROOT_LIBDIR}/libfakeroot/${LIB_TARGET}" ] ; then
			warn "fakeroot: libfakeroot-${SBOX_FAKEROOT_MODE} not found from toolchain, falling back to tcp"
			LIB_HOST='libfakeroot-host-tcp.so'
			LIB_TARGET='libfakeroot-tcp.so.0'
			SBOX_FAKEROOT_MODE='tcp'
		fi
	fi
fi

PATHS=${LIB_PREFIX}/lib/libfakeroot:${LIB_PREFIX}/lib/libfakeroot-${SBOX_FAKEROOT_MODE}
FAKED="${FAKEROOT_BINDIR}/faked-${SBOX_FAKEROOT_MODE}"

FAKED_MODE="unknown-is-root"
export FAKED_MODE

libfound=no

GETOPTEST=`getopt --version`
case $GETOPTEST in
getopt*) # GNU getopt
    FAKE_TEMP=`getopt -l lib: -l faked: -l unknown-is-real -l fd-base: -l version -l help -- +l:f:i:s:ub:vh "$@"`
    ;;
*) # POSIX getopt ?
    FAKE_TEMP=`getopt l:f:i:s:ub:vh "$@"`
    ;;
esac

if test "$?" -ne 0; then
  usage
fi

eval set -- "$FAKE_TEMP"

FAKEDOPTS=""
PIPEIN=""
WAITINTRAP=0

while test "X$1" != "X--"; do
  case "$1" in
    -l|--lib)
       shift
       LIB_TARGET=`eval echo "$1"`
       PATHS=
       ;;
    -f|--faked)
       shift
       FAKED="$1"
       ;;
    -i)
       shift
       if test -f "$1"; then
         FAKEDOPTS=$FAKEDOPTS" --load"
         PIPEIN="<$1"
       else
         stderr "database file \`$1' does not exist."
       fi
       ;;
    -s)
       shift
       FAKEDOPTS=$FAKEDOPTS" --save-file $1"
       [ -p $1 ] || WAITINTRAP=1
       ;;
    -u|--unknown-is-real)
       FAKEDOPTS=$FAKEDOPTS" --unknown-is-real"
       FAKED_MODE="unknown-is-real"
       ;;
    -b|--fd-base)
       shift
       FAKEROOT_FD_BASE="$1"
       ;;
    -v|--version)
       echo "fakeroot version 1.14.4 (scratchbox)"
       exit 0
       ;;
    -h|--help)
       usage
       ;;
  esac
  shift
done

shift # get rid of the '--'

# make sure the preload is available
if [ -n "$PATHS" ] ; then
	for dir in `echo $PATHS | sed 's/:/ /g'` ; do
		if [ -r "${dir}/${LIB_TARGET}" ] ; then
			libfound=yes
		fi
	done
else
	if [ -r "$LIB_TARGET" ] ; then
		libfound=yes
	fi
fi

if test $libfound = no
then
    fatal "preload library \`$LIB_TARGET' not found, aborting."
fi

if test -n "$FAKEROOTKEY"
then
    fatal "FAKEROOTKEY set to $FAKEROOTKEY" \
          "nested operation not yet supported"
fi

unset FAKEROOTKEY
KEY_PID=`eval $FAKED $FAKEDOPTS $PIPEIN`
FAKEROOTKEY=`echo $KEY_PID|cut -d: -f1`
PID=`echo $KEY_PID|cut -d: -f2`

# return missing preload
if [ "z$LD_PRELOAD" = 'z' ] ; then
  LD_PRELOAD="${LIB_TARGET}" 
else
  LD_PRELOAD="${LIB_TARGET} ${LD_PRELOAD}"
fi

if [ "$WAITINTRAP" -eq 0 ]; then
  trap "kill -s TERM $PID" EXIT INT
else
  trap 'FAKEROOTKEY=$FAKEROOTKEY LD_LIBRARY_PATH="$PATHS" SBOX_PRELOAD=${HOST_PREFIX}/lib/libfakeroot/${LIB_HOST} LD_PRELOAD="$LD_PRELOAD" kill -s TERM $PID ; while kill -0 $PID >/dev/null 2>&1 ; do sleep 0.1 ; done' EXIT INT
fi

if test -z "$FAKEROOTKEY" || test -z "$PID"; then
  fatal "error while starting the \`faked' daemon."
fi

export FAKEROOT_FD_BASE

export _SBOX_NONFAKE_UID=`id -u`
export _SBOX_NONFAKE_GID=`id -g`

if test -z "$*"; then
  FAKEROOTKEY=$FAKEROOTKEY LD_LIBRARY_PATH="$PATHS" SBOX_PRELOAD="${HOST_PREFIX}/lib/libfakeroot/${LIB_HOST},${LIB_TARGET}" LD_PRELOAD="$LD_PRELOAD" ${SHELL:-/bin/sh}
  RESULT=$?
else
  FAKEROOTKEY=$FAKEROOTKEY LD_LIBRARY_PATH="$PATHS" SBOX_PRELOAD="${HOST_PREFIX}/lib/libfakeroot/${LIB_HOST},${LIB_TARGET}" LD_PRELOAD="$LD_PRELOAD" "$@"
  RESULT=$?
fi

exit $RESULT

# Local Variables:
# mode: shell-script
# End:
